(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs'), require('rxjs/operators'), require('@angular/common/http')) :
    typeof define === 'function' && define.amd ? define('ngx-operators', ['exports', 'rxjs', 'rxjs/operators', '@angular/common/http'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ngx-operators'] = {}, global.rxjs, global.rxjs.operators, global.ng.common.http));
}(this, (function (exports, rxjs, operators, http) { 'use strict';

    /**
     * Invokes a callback upon subscription.
     *
     * @param callback function to invoke upon subscription
     * @returns stream which will invoke callback
     */
    function prepare(callback) {
        return function (source) { return rxjs.defer(function () {
            callback();
            return source;
        }); };
    }

    /**
     * Indicates whether the observable is currently loading (meaning subscription is active and
     * it hasn't completed or errored).
     *
     * @param indicator subject as target for indication
     * @returns stream which will indicate loading through passed subject
     */
    function indicate(indicator) {
        return function (source) { return source.pipe(prepare(function () { return indicator.next(true); }), operators.finalize(function () { return indicator.next(false); })); };
    }

    /**
     * Maps Angular HTTP status codes to more semantic errors.
     *
     * @param codeErrors mapping from status code to error
     * @returns stream which will map http error codes to passed errors
     */
    function throwForCodes(codeErrors) {
        return function (source) { return source.pipe(operators.catchError(function (error) {
            var _a, _b;
            if (error instanceof http.HttpErrorResponse) {
                return rxjs.throwError((_b = (_a = codeErrors[error.status]) === null || _a === void 0 ? void 0 : _a.call(codeErrors)) !== null && _b !== void 0 ? _b : error);
            }
            return rxjs.throwError(error);
        })); };
    }

    function isHttpResponse(event) {
        return event.type === http.HttpEventType.Response;
    }
    function isHttpProgressEvent(event) {
        return (event.type === http.HttpEventType.DownloadProgress ||
            event.type === http.HttpEventType.UploadProgress);
    }

    function download(saver) {
        return function (source) { return source.pipe(operators.scan(function (last, event) {
            if (isHttpProgressEvent(event)) {
                return {
                    progress: event.total
                        ? Math.round((100 * event.loaded) / event.total)
                        : last.progress,
                    state: "IN_PROGRESS",
                    content: null
                };
            }
            if (isHttpResponse(event)) {
                if (saver && event.body) {
                    saver(event.body);
                }
                return {
                    progress: 100,
                    state: "DONE",
                    content: event.body
                };
            }
            return last;
        }, { state: "PENDING", progress: 0, content: null }), operators.distinctUntilChanged(function (a, b) { return a.state === b.state &&
            a.progress === b.progress &&
            a.content === b.content; })); };
    }

    /**
     * If the underlying request throws with a 404 error, it will swallow the error
     * and complete the observable instead. All other errors will be rethrown.
     *
     * You can then set a fallback value with `defaultIfEmpty`.
     *
     * @returns stream which will map 404 errors to `undefined`
     * @example
     * this.http.get<User>('/users/123').pipe(
     *   ignoreNotFound()
     * )
     */
    function ignoreNotFound() {
        return function (source) { return source.pipe(operators.catchError(function (error) {
            if (error instanceof http.HttpResponseBase && error.status === 404) {
                return rxjs.EMPTY;
            }
            return rxjs.throwError(error);
        })); };
    }

    var noZoneRunner = function (zone) { return function (source) {
        return new rxjs.Observable(function (subscriber) {
            return zone.runOutsideAngular(function () {
                runInZone(zone)(source).subscribe(subscriber);
            });
        });
    }; };
    function runOutsideZone(zone) {
        return function (source) {
            return new rxjs.Observable(function (subscriber) {
                return source.subscribe(function (value) { return zone.runOutsideAngular(function () { return subscriber.next(value); }); }, function (e) { return zone.runOutsideAngular(function () { return subscriber.error(e); }); }, function () { return zone.runOutsideAngular(function () { return subscriber.complete(); }); });
            });
        };
    }
    function runInZone(zone) {
        return function (source) {
            return new rxjs.Observable(function (subscriber) {
                return source.subscribe(function (value) { return zone.run(function () { return subscriber.next(value); }); }, function (e) { return zone.run(function () { return subscriber.error(e); }); }, function () { return zone.run(function () { return subscriber.complete(); }); });
            });
        };
    }

    function upload() {
        var initialState = { state: "PENDING", progress: 0 };
        var reduceState = function (state, event) {
            if (isHttpProgressEvent(event)) {
                return {
                    progress: event.total
                        ? Math.round((100 * event.loaded) / event.total)
                        : state.progress,
                    state: "IN_PROGRESS",
                };
            }
            if (isHttpResponse(event)) {
                return {
                    progress: 100,
                    state: "DONE",
                };
            }
            return state;
        };
        return function (source) { return source.pipe(operators.scan(reduceState, initialState), operators.distinctUntilChanged(function (a, b) { return a.state === b.state && a.progress === b.progress; })); };
    }

    /*
     * Public API Surface of ngx-operators
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.download = download;
    exports.ignoreNotFound = ignoreNotFound;
    exports.indicate = indicate;
    exports.noZoneRunner = noZoneRunner;
    exports.prepare = prepare;
    exports.runInZone = runInZone;
    exports.runOutsideZone = runOutsideZone;
    exports.throwForCodes = throwForCodes;
    exports.upload = upload;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-operators.umd.js.map
